/*
 * Svjatoslav Commons - shared library of common functionality.
 * Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 */

package eu.svjatoslav.commons.commandline.parameterparser.parameter;

import eu.svjatoslav.commons.commandline.parameterparser.ArgumentCount;
import eu.svjatoslav.commons.commandline.parameterparser.Parameter;

import java.io.File;

public class FileParameter extends Parameter<File, FileParameter> {

    private ExistenceType existenceType = ExistenceType.DOES_NOT_MATTER;

    public FileParameter(final String description) {
        super(description, ArgumentCount.SINGLE);
    }

    @Override
    public java.lang.String describeFormat() {
        return existenceType.description + " file";
    }

    @Override
    public File getValue() {

        if (arguments.size() != 1)
            throw new RuntimeException("Parameter " + description
                    + " shall have exactly 1 argument.");

        return new File(arguments.get(0));
    }

    public FileParameter mustExist() {
        existenceType = ExistenceType.MUST_EXIST;
        return this;
    }

    public FileParameter mustNotExist() {
        existenceType = ExistenceType.MUST_NOT_EXIST;
        return this;
    }

    @Override
    public boolean validate(final java.lang.String value) {
        final File file = new File(value);

        if (existenceType == ExistenceType.MUST_EXIST) {
            return file.exists() && file.isFile();
        }

        if (existenceType == ExistenceType.MUST_NOT_EXIST) {
            return !file.exists();
        }

        if (existenceType == ExistenceType.DOES_NOT_MATTER) {
            if (file.exists())
                if (file.isDirectory())
                    return false;

            return true;
        }

        return false;
    }

}
