/*
 * Svjatoslav Commons - shared library of common functionality.
 * Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 */

package eu.svjatoslav.commons.network.navigation;

import eu.svjatoslav.commons.network.LocaleConfiguration;

import javax.servlet.http.HttpServletRequest;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.List;

public class Navigation<NI extends NavigationItem> {

    protected final LocaleConfiguration localeConfiguration;
    private NI rootNavigationItem;
    private NI defaultNavigationItem;

    public Navigation(final LocaleConfiguration localeConfiguration) {
        this.localeConfiguration = localeConfiguration;
    }

    public NI getDefaultNavigationItem() {
        return defaultNavigationItem;
    }

    private String getGetNotNullString(final String string) {
        if (string != null)
            return string;

        return "";
    }

    public LocaleConfiguration getLocaleConfiguration() {
        return localeConfiguration;
    }

    public NI getRootNavigationItem() {
        return rootNavigationItem;
    }

    public NI getSelectedItem(final HttpServletRequest request) {
        final String requestUrl = request.getRequestURL().toString();

        try {
            final String requestPath = new URL(requestUrl).getPath();

            @SuppressWarnings("unchecked") final NI match = (NI) rootNavigationItem.getMatchingNavigationItem(requestPath);

            if (match != null)
                return match;
        } catch (final MalformedURLException e) {
            System.out.println("Cannot handle URL: " + requestUrl);
            e.printStackTrace();
        }

        return getDefaultNavigationItem();
    }

    @SuppressWarnings("unchecked")
    public String getTopMenu(final HttpServletRequest request) {

        final String currentLocale = localeConfiguration.detectCurrentLocale(request);

        final NI selectedItem = getSelectedItem(request);
        final StringBuilder result = new StringBuilder();

        result.append("<div class=\"" + getGetNotNullString(rootNavigationItem.getCssPrefix()) + "navigationMenu\">");

        for (final NI item : (List<NI>) getRootNavigationItem().getSubElements()) {

            if (item == selectedItem)
                result.append("<a class=\"" + getGetNotNullString(item.getCssPrefix()) + "menuItemSelected\" href=\""
                        + item.getLinkUrl() + "\">");
            else
                result.append("<a class=\"" + getGetNotNullString(item.getCssPrefix()) + "menuItem\" href=\""
                        + item.getLinkUrl() + "\">");

            result.append("<div>" + item.getTitle(currentLocale) + "</div>");
            result.append("</a>");
        }

        result.append("</div>");

        return result.toString();
    }

    public NI setDefaultNavigationItem(final NI defaultNavigationItem) {
        this.defaultNavigationItem = defaultNavigationItem;
        return defaultNavigationItem;
    }

    public NI setRootNavigationItem(final NI rootNavigationItem) {
        this.rootNavigationItem = rootNavigationItem;
        return rootNavigationItem;
    }

}
