/*
 * Svjatoslav Commons - shared library of common functionality.
 * Copyright ©2012-2017, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 */

package eu.svjatoslav.commons.string;

import org.junit.Test;

import static org.junit.Assert.assertEquals;

public class String2Test {

    @Test
    public void testTrimPrefix() {

        final String2 s = new String2("this is a test");

        assertEquals("is a test", s.trimPrefix(5).toString());
        assertEquals("a test", s.trimPrefix(3).toString());
        assertEquals("test", s.trimPrefix(2).toString());
        assertEquals("", s.trimPrefix(500).toString());
    }

    @Test
    public void testTrimSuffix() {

        final String2 s = new String2("this is a test");

        assertEquals("this is a", s.trimSuffix(5).toString());
        assertEquals("this is", s.trimSuffix(2).toString());
        assertEquals("this", s.trimSuffix(3).toString());
        assertEquals("", s.trimSuffix(500).toString());
    }


    @Test
    public void testEnforceLength() {
        final String2 s = new String2("12345678");
        s.enforceLength(3);
        assertEquals("123", s.toString());

        s.enforceLength(5);
        assertEquals("123  ", s.toString());
    }

    @Test
    public void testSuffixAndPrefix() {
        final String2 s = new String2("experiment");
        s.addPrefix("The ").addSuffix(" !");

        assertEquals("The experiment !", s.toString());
    }

}
